/** @file
  Jansson private configurations for UEFI support.

 Copyright (c) 2019, Intel Corporation. All rights reserved.<BR>
 (C) Copyright 2020 Hewlett Packard Enterprise Development LP<BR>

    SPDX-License-Identifier: BSD-2-Clause-Patent

**/

#ifndef JANSSON_PRIVATE_CONFIG_H_
#define JANSSON_PRIVATE_CONFIG_H_

#define HAVE_SYS_TIME_H   1
#define HAVE_SYS_TYPES_H  1

#define INITIAL_HASHTABLE_ORDER  3

#endif
