/** @file
  InstrinsicLib class with intrinsic APIs generated by compilers.

  Copyright (c) 2022, Intel Corporation. All rights reserved.<BR>
  SPDX-License-Identifier: BSD-2-Clause-Patent

**/

#ifndef INTRINSTIC_LIB_H_
#define INTRINSTIC_LIB_H_

//
// Compiler dependent intrinsic APIs.
//

#endif
