/** @file
  OpensslLib class with APIs from the openssl project

  Copyright (c) 2022, Intel Corporation. All rights reserved.<BR>
  SPDX-License-Identifier: BSD-2-Clause-Patent

**/

#ifndef OPENSSL_LIB_H_
#define OPENSSL_LIB_H_

#include <openssl/opensslv.h>

#endif
