/** @file
  Include file to support building the third-party jansson library.

 Copyright (c) 2019, Intel Corporation. All rights reserved.<BR>
 (C) Copyright 2020 Hewlett Packard Enterprise Development LP<BR>

    SPDX-License-Identifier: BSD-2-Clause-Patent

**/

#ifndef REDFISH_CRT_STDARG_H_
#define REDFISH_CRT_STDARG_H_

#include <Library/RedfishCrtLib.h>

#endif
